--v1.8a

local function formatStat(name, value, color, tooltip)
	if value == "" or value == nil then
		return ""
	end

	if color == nil then
		color = "black"
	end
		
	local toolTipHTML = ""

	if tooltip then
		toolTipHTML = "<span class='tooltip'>"..tooltip.."</span>"
	end
	
	return "<div class='stat' style='color:".. color ..";'><span style='margin-right:5px;'>" .. name .. ":</span><span>" .. value .. "</span>"..toolTipHTML.."</div>"
end

local function formatMonsterItems(monster)
	local data = monster.Items
	if not data then return " " end

	local itemdatalist = {}
	for value in table.values(data) do
		local dropinfo = ""
		
		if value["unknown drop rate"] then
			dropinfo = "??"
		else
			local chance = tonumber(value.Chance) or 0
			if chance > 0 then
				dropinfo = chance .. "%"
			end
		end
		
		if value["conditional"] then
			dropinfo = dropinfo .. " (conditional)"
		end
		
		if value["pickpocket only"] then
			dropinfo = dropinfo .. " (pickpocket only)"
		end
		
		if value["no pickpocket"] then
			dropinfo = dropinfo .. " (no pickpocket)"
		end
			
		if value["bounty"] then
			dropinfo = dropinfo .. " (bounty)"
		end
			
		local itemData = "<div>"
		itemData = itemData .. "<span style='margin-right:5px;'>"..value.Name .. ":</span>"
		itemData = itemData .. "<span>" .. dropinfo .. "</span></div>"
		table.insert(itemdatalist, itemData)
	end
	local itemDataText = "<div style='float:left;'>"
	itemDataText = itemDataText .. "<div style='font-size:12px;'>Drops</div>"
	itemDataText = itemDataText .. table.concat(itemdatalist)
	itemDataText = itemDataText .. "</div>"
	return itemDataText
end

local elementColor = {
	["stench"] = "green",
	["hot"] = "red",
	["cold"] = "blue",
	["sleaze"] = "blueviolet",
	["spooky"] = "grey",
}

local phlyumFamiliars = {
	[150] = true, -- stomping boots
	[159] = true, -- happy medium
}

local function formatMonsterStats(monster)
	local data = monster.Stats
	if not data then return " " end

	local color
	if data.Element then
		color = elementColor[data.Element]
	end

	local statData = "<div style='margin-right:10px;float:left;'>"
	statData = statData .. "<div style='font-size:12px;'>Stats</div>"
	statData = statData .. formatStat("HP", data.ModHP, nil, "Starting HP: " .. data.HP)
	statData = statData .. formatStat("Atk", data.ModAtk, nil, "Starting Atk: " .. data.Atk)
	
	local defTooltip = data.Def
	if data.Def ~= "?" and tonumber(data.Def) then
		defTooltip = math.max(math.floor(tonumber(data.Def) * 0.9), 1)
	end
	
	statData = statData .. formatStat("Def", data.ModDef, nil, "Starting Def: " .. defTooltip)
	statData = statData .. formatStat("Meat", data.Meat)
	statData = statData .. formatStat("Element", data.Element, color)
	statData = statData .. formatStat("Init", data.Init)

	if phlyumFamiliars[familiarid()] then	
		local statField = [[<div onclick='javascript:togglePhylum();' style='cursor:hand;' class='stat'><span style='margin-right:5px;'><span id='phylumToggle'>[+]</span> Phylum:</span><span>]] .. data.Phylum .. "</span></div>"			
		statData = statData .. statField
	end	
	
	statData = statData .. formatStat("PhyRes", data.PhyRes)
	statData = statData .. formatStat("Watch out for", data.WatchOut)
	statData = statData .. "</div>"
	return statData
end

local function formatMonsterPhylumTreasure(monster)
	local data = monster.Stats
	local html = ""
	
	local treasure = get_phylum_treasure(data.Phylum)
	
	local function effecTurnsString(name, duration)
		if name ~= "none" then
			return "Effect: " .. name .. " (" .. duration .. " turns)"  
		else
			return ""
		end
	end
	
	if phlyumFamiliars[familiarid()] then
		html = [[<div style='display:none;font-size:12px;color:#666;margin:0px auto;text-align:left;' id='phylumData'>]]
	end
			
	if familiarid() == 150 then
		treasure = treasure["stomp"]
		
		html = html .. [[<div style='width:200px;margin:0px auto;'>]]..
			"Release the Boots will generate:<br/><br/>" ..			
			treasure["name"] .. "<br />" ..
			"Level required: " .. treasure["quality"]["level requirement"].."<br />" ..
			"Adventures: " .. treasure["quality"]["adventures"].."<br />" ..
			effecTurnsString(treasure["effect"]["name"], treasure["quality"]["effect duration"]) .. "<br />"
		
		if treasure["effect"]["summary"] ~= "none" then
			html = html.."Summary: " .. treasure["effect"]["summary"]
		end
		
		html = html .. [[</div>]]
		
	end
	
	if familiarid() == 159 then
		treasure = treasure["siphon"]
		local function generateSiphonBooze(boozeData)
			return [[<div style="float:left;margin-right:10px;width:200px;">]]..
				"<span>"..boozeData["name"].."</span><br />" ..
				"Type: " .. boozeData["quality"]["quality"].."<br />" ..
				"Level required: " .. boozeData["quality"]["level requirement"].."<br />" ..
				"Adventures: " .. boozeData["quality"]["adventures"].."<br />" ..
				effecTurnsString(boozeData["effect"]["name"], boozeData["quality"]["effect duration"]) .. "<br />" ..
				"Summary: " .. boozeData["effect"]["summary"]..
				[[</div>]]
		end
	
		html = html..
			"Siphon Spirits will generate:<br/><br/>"..
			generateSiphonBooze(treasure["blue"]) ..
			generateSiphonBooze(treasure["orange"]) ..
			generateSiphonBooze(treasure["red"])
	end
	
	if phlyumFamiliars[familiarid()] then
		html = html .. [[<p style="clear:both;"></p></div>]]
	end
			
	
	return html
end

--TODO: Resolve as many ambiguous names as you can find
-- orcish frat boy
-- slimetube
-- hobopolis 
-- more...
local ambiguousNames = {
	[ [[animated nightstand]] ] = { 
		[ [[darkstand.gif]] ] = [[animated nightstand (mahogany)]],
		[ [[nightstand.gif]] ] =[[animated nightstand (white)]],
	},
	[ [[ed the undying]] ] = { 
		[ [[ed.gif]] ] = [[ed the undying (1)]],
		[ [[ed2.gif]] ] = [[ed the undying (2)]],
		[ [[ed3.gif]] ] = [[ed the undying (3)]],
		[ [[ed4.gif]] ] = [[ed the undying (4)]],
		[ [[ed5.gif]] ] = [[ed the undying (5)]],
		[ [[ed6.gif]] ] = [[ed the undying (6)]],
		[ [[ed7.gif]] ] = [[ed the undying (7)]],
	},
	[ [[ninja snowman]] ] = {
		[ [[snowman.gif]] ] = [[ninja snowman (hilt/mask)]],
		[ [[ninjarice.gif]] ] =[[ninja snowman (chopsticks)]],
	},
	[ [[knight]] ] = {
		[ [[snaknight.gif]] ] = [[knight (snake)]],
		[ [[wolfknight.gif]] ] = [[knight (wolf)]],
	}
}

local function parseMonsterNameIntoMafiaFormat(monsterName, fightText)
	--mafia's table doesn't precede the names of the monsters
	--with "a " or "an " or spaces, this function attempts to
	--strip out those things
	--mafia's table was also inconsistent with cases, 
	--so I lowercased everything and will have to do so here as well
	
	local mafiaMonsterName = monsterName:lower()

	mafiaMonsterName = mafiaMonsterName:gsub("^a ", "")
	mafiaMonsterName = mafiaMonsterName:gsub("^an ", "")
	mafiaMonsterName = mafiaMonsterName:gsub("^ ", "")

	if ambiguousNames[mafiaMonsterName] then
		for lookFor, actualName in pairs(ambiguousNames[mafiaMonsterName]) do
			if fightText:contains(lookFor) then
				return actualName
			end
		end	
	end

	return mafiaMonsterName
end

local function beesIncreaser(monster_name, base_data)
	local bees = {
		[" beebee gunners"] = true,
		["a moneybee"] = true,
		["a mumblebee"] = true,
		["a beebee queue"] = true,
		["a bee swarm"] = true,
		["a buzzerker"] = true,
		["a Beebee King"] = true,
		["a bee thoven"] = true,
		["a Queen Bee"] = true,
	}

	local _,numberBees = monster_name:gsub("[bB]", "%0")
	if numberBees > 0 and not bees[monster_name] then
		local modifier = 1 + (numberBees * .2)
		base_data.Stats.HP = base_data.Stats.HP * modifier
		base_data.Stats.Atk = base_data.Stats.Atk * modifier
		base_data.Stats.Def = base_data.Stats.Def * modifier
	end
	
	return base_data
end

local function adjustFromCombat(monster)
	
	if monster.Stats.HP ~= "?" then
		monster.Stats.ModHP = (tonumber(monster.Stats.HP)) - (tonumber(fight["damage inflicted"]) or 0)
	else
		if fight["damage inflicted"] ~= "0" then
			monster.Stats.ModHP = tonumber(fight["damage inflicted"]) * -1
		else
			monster.Stats.ModHP = monster.Stats.HP
		end
	end
	
	if monster.Stats.Atk ~= "?" then
		monster.Stats.ModAtk = math.max(1,(tonumber(monster.Stats.Atk)) - (tonumber(fight["attack decrease"]) or 0))
	else
		if fight["attack decrease"] ~= "0" then
			monster.Stats.ModAtk = tonumber(fight["attack decrease"]) * -1
		else
			monster.Stats.ModAtk = monster.Stats.Atk
		end
	end
	
	if monster.Stats.Def ~= "?" then
		monster.Stats.ModDef = math.max(1,math.floor(tonumber(monster.Stats.Def) * 0.9) - (tonumber(fight["defense decrease"]) or 0))
	else
		if fight["defense decrease"] ~= "0" then
			monster.Stats.ModDef = math.floor(tonumber(fight["defense decrease"]) * 0.9) * -1
		else
			monster.Stats.ModDef = monster.Stats.Def
		end		
	end
		
	return monster.Stats
end

local function getMonsterData(monsterName, fightText)
	local mlmods = estimate_ML_modifiers()
	local ml = 0
	for _, m in pairs(mlmods) do
		ml = ml + m
	end

	local itemmods = estimate_item_modifiers()
	local item = 0
	for _, m in pairs(itemmods) do
		item = item + m
	end

	local monsterDataName = parseMonsterNameIntoMafiaFormat(monsterName, fightText)
	if monsterDataName == "tomb rat king" then ml = 0 end -- ML doesn't get reapplied when using rat tangles
	local base_data = get_monster_base_data(monsterDataName)
	local ml_increases = {
		HP = true,
		Atk = true,
		Def = true,
	}
	
	if base_data then
		local monster = {}
		if base_data.Stats then
			monster.Stats = {}
			for a, b in pairs(base_data.Stats) do
				if b == 0 then
					monster.Stats[a] = "?"
				elseif ml_increases[a] then
					monster.Stats[a] = math.max(b + ml, 1)
				else
					monster.Stats[a] = b
				end
			end
			
			--In a bees hate you, monster's with b in their names get increased by 20% per b
			--This is AFTER ML is applied
			if ascensionpathid() == 4 then
				monster.Stats = beesIncreaser(monsterName, monster.Stats)
			end
	
			monster.Stats = adjustFromCombat(monster)
		end
		if base_data.Items then
			monster.Items = {}
			for idata in table.values(base_data.Items) do
				local newidata = {}
				for a, b in pairs(idata) do
					if a == "Chance" then
						local p_base = b / 100
						local p_mod = p_base * (1 + item / 100)
						p_mod = math.max(math.min(p_mod, 1), 0)
						newidata[a] = round_down(p_mod * 100, 2)
					else
						newidata[a] = b
					end
				end
				table.insert(monster.Items, newidata)
			end
		end
		return monster
	else
		return nil
	end
end

add_printer("/fight.php", function()
	if character["show monster stats"] == "yes" then
		
		text = text:gsub([[</head>]], [[
<style>
	.stat{
		position:relative;
	}
	
	.stat .tooltip{
		border:1px solid #666;
		color:#666;
		background-color:#f5f5f5;
		padding:3px;
		display:none;
		border-radius:2px;
	}
	
	.stat:hover .tooltip{
		display:block;
		position: absolute;
		left:10px;
		top:17px;
		z-index:99;
		width:100px;
		box-shadow: 3px 3px 4px #777;
	}
</style>
<script type="text/javascript">
	function togglePhylum(){
		$('#phylumData').slideToggle();
		
		if($('#phylumToggle').html() == '[+]')
			$('#phylumToggle').html('[-]')
		else
			$('#phylumToggle').html('[+]')
	}
</script>%0]])		
		local monster = getMonsterData(monster_name, text)
		if monster then
			text = text:gsub([[<td><img id='monpic']], [[<td style='vertical-align:top;' rowspan=2><img id='monpic']])	
			text = text:gsub([[(id='monname'.-)</td></tr></table><br>]], function(prefix)
				return prefix .. [[</td></tr><tr><td><div style='font-size:11px;color:#555;'>]] .. formatMonsterStats(monster) .. formatMonsterItems(monster) .. [[</div></td></tr></table><br>]]
			end)
			
			text = text:gsub([[</td></tr></table><br>]], [[%0]]..formatMonsterPhylumTreasure(monster)..[[<br />]])
									
			if monster.Stats.Phylum then
				text = text:gsub([[img id='monpic']], [[%0 title='Phylum: ]]..monster.Stats.Phylum..[[']])
			end
			
			if ascensionpathid() == 4 then
				local boldedName = monster_name:gsub("[bB]", [[<span style="font-weight:bold;color:orange;">%0</span>]])
				text = text:gsub(monster_name, boldedName)
			end
		end
	end
	
end)

add_processor("/fight.php", function()
	if text:contains("You slap a flyer up on your opponent") then
		local flyertext = text:match([[documents.gif" width=30 height=30 alt=.-You slap a flyer up on your opponent]])
		local whichflyer = flyertext:match([[title="(.-)"]])
		if whichflyer then
			local monster = getMonsterData(monster_name, text)
			if monster and monster.Stats then
				local atk = tonumber(monster.Stats.Atk)
				if atk then
					if whichflyer == "rock band flyers" then
						increase_ascension_counter("zone.island.frat arena flyerML", atk)
					elseif whichflyer == "jam band flyers" then
						increase_ascension_counter("zone.island.hippy arena flyerML", atk)
					end
				end
			end
		end
	end
end)

local function getFightBody(htmlPageText)
	--get fight body
	local pos,_ = htmlPageText:find("<span id='monname'>")
	part1 = htmlPageText:sub(pos)
	
	local _,pos1 = part1:find("</span>")
	part1 = part1:sub(pos1+1)
	
	local pos2,_=part1:find("<div id='fightform'")
	part2 = part1:sub(0, pos2-1)
	--remove monster intro text
	part2 = part2:gsub([[<blockquote>(.-)</blockquote>]], "")
	return part2
end
	
local blackList ={
	[[%b<>]], --any html tags
	[[You lose %d+]], --damage FROM monsters
	[[reduced by %d+]], --develers
	[[You gain %d+]], --hp or mp gain during combat
	[[macroaction: skill %d+]], --macro ids
	[[%d+ scroll]], -- scroll combat items
	[[duration: %d+]], -- acquiring an effect during combat
	[[stabs you for %d+ damage]], --stab bat
	[[sown %d+ damage]], --ice sickle, we'll get the damage on the reap message
	[[you strain your neck, doing %d+ damage]], --birdform fumble
}

local function stripOutBlackList(fightBodyText)
	for item in table.values(blackList) do
		fightBodyText = fightBodyText:gsub(item, "")
	end
	
	return fightBodyText
end

local mainPattern =" [0-9()+%s]+ " --this is the main damage pattern 
							 --it WILL need a suffix otherwise it will capture too many things

local whiteList = {
	mainPattern..[[damage]], --main pattern, this will cover most of the things
	[[transferring %d+ points of]], --ggg
	[[%d+ months worth of concentrated palm sweat]], --stinkpalm
	[[%d+ HP worth]], --scary death orb
	mainPattern..[[tiny holes into him]], --birdform
	mainPattern..[[whacks to the face]], --Ax of L'rose
	mainPattern..[[points worth of blood ooze]], -- all clubs
	[[dealing]]..mainPattern..[[to your opponent,]], -- asparagus knife
	mainPattern..[[bullets]], -- astral pistol
	mainPattern..[[hollow]], -- astral pistol
	[[spear factor by]]..mainPattern..[[points]], -- prehistoric spear
	mainPattern..[[points of really neat, groovy, and hip PAIN]], -- cool whip
	mainPattern..[[points' worth, to be precise]], -- disco ball
	mainPattern..[[points' worth of naughtiness]], -- origami riding crop
	mainPattern..[[to the foul demon]], -- sing against AT nemesis
}

local function getCombatDamage(fightBodyText)
	filteredText = stripOutBlackList(fightBodyText)
		
	local total = 0;
	
	for item in table.values(whiteList) do
		for number in filteredText:gmatch(item) do
			for num in number:gmatch([[%d+]]) do
				total = total + tonumber(num)
			end
		end
		filteredText = filteredText:gsub(item, "", 1)
	end

	return total	
end

add_processor("/fight.php", function()
	fightBody = getFightBody(text)

	local total = getCombatDamage(fightBody)
	
	local damage = tonumber(fight["damage inflicted"]) or 0
	damage = damage + total
	
	fight["damage inflicted"] = damage
	
	--atk delevel
	local atkDelevel = 0
	for number in fightBody:gmatch([[Monster attack power reduced by <b>%d+]]) do
		number = number:match([[%d+]])
		atkDelevel = atkDelevel + tonumber(number)
	end
	local totalAtkDelevel = tonumber(fight["attack decrease"]) or 0
	totalAtkDelevel = totalAtkDelevel + atkDelevel
	fight["attack decrease"] = totalAtkDelevel
	
	--def delevel
	local defDelevel = 0
	for number in fightBody:gmatch([[Monster defense reduced by <b>%d+]]) do
		number = number:match([[%d+]])
		defDelevel = defDelevel + tonumber(number)
	end
	local totalDefDelevel = tonumber(fight["defense decrease"]) or 0
	totalDefDelevel = totalDefDelevel + defDelevel
	fight["defense decrease"] = totalDefDelevel
end)

add_printer("/fight.php", function()
	if text:contains("You slap a flyer up on your opponent") then
		text = text:gsub([[documents.gif" width=30 height=30 alt=.-</td></tr>]], function(flyertext)
			local completion = "?"
			local whichband = "?"
			if flyertext:contains([[title="rock band flyers"]]) then
				completion = round_down((ascension["zone.island.frat arena flyerML"] or 0) / 100, 1)
				whichband = "rock band"
			elseif flyertext:contains([[title="jam band flyers"]]) then
				completion = round_down((ascension["zone.island.hippy arena flyerML"] or 0) / 100, 1)
				whichband = "jam band"
			end
			return flyertext:gsub([[(You slap a flyer up on your opponent.-)(<)]], [[%1 <span style="color:green">{ Advertised ~]] .. completion .. [[%% for ]]..whichband..[[. }</span>%2]])
		end)
	end
end)

add_printer("/bigisland.php", function()
	if have("rock band flyers") or have("jam band flyers") then
		local msgs = {}
		
		if have("rock band flyers") then
			table.insert(msgs, "{ ~" .. (ascension["zone.island.frat arena flyerML"] or 0) .. " ML slapped for frat boys. }")
		end
		if have("jam band flyers") then
			table.insert(msgs, "{ ~" .. (ascension["zone.island.hippy arena flyerML"] or 0) .. " ML slapped for hippies. }")
		end
		text = text:gsub([[<p><Center>]], [[<p style="color:green;text-align:center;">]] .. table.concat(msgs, "<br>\n") .. [[</p>%0]])
	end
end)

add_printer("/inventory.php", function()
	local fratMLCompleted = ascension["zone.island.frat arena flyerML"] or 0
	local hippyMLCompleted = ascension["zone.island.hippy arena flyerML"] or 0
	
	text = text:gsub([[rock band flyers]], [[%0&nbsp;]]..[[<span style="color:green;" >{]]..fratMLCompleted..[[ ML slapped}</span>]])
	text = text:gsub([[jam band flyers]], [[%0&nbsp;]]..[[<span style="color:green;" >{]]..hippyMLCompleted..[[ ML slapped}</span>]])
end)
